from coralillo.datamodel import debyte_set


class PermissionHolder:

    def allow_key(self):
        ''' Gets the key associated with this user where we store permission
        information '''
        return self.key() + ':allow'

    def allow(self, objspec):
        engine = type(self).get_engine()

        return engine.lua.allow(keys=[self.allow_key()], args=[objspec])

    def is_allowed(self, objspec, *, tail=None):
        engine = type(self).get_engine()

        return engine.lua.is_allowed(keys=[self.allow_key()], args=[objspec, tail])

    def revoke(self, objspec):
        engine = type(self).get_engine()

        return engine.redis.srem(self.allow_key(), objspec)

    def get_perms(self):
        engine = type(self).get_engine()

        return debyte_set(engine.redis.smembers(self.allow_key()))



from coralillo import Model, Engine
from coralillo.fields import Text, SetRelation, ForeignIdRelation

nrm = Engine()


# For one-many relations
class Pet(Model):
    name = Text()
    owner = ForeignIdRelation('coralillo.tests.models.Person', inverse='pets')

    class Meta:
        engine = nrm


class Person(Model):
    name = Text()
    pets = SetRelation(Pet, on_delete='cascade', inverse='owner')

    class Meta:
        engine = nrm


# For many to many relations
class Driver(Model):
    name = Text()
    cars = SetRelation('coralillo.tests.models.Car', inverse='drivers')

    class Meta:
        engine = nrm


class Car(Model):
    name = Text()
    drivers = SetRelation(Driver, inverse='cars')

    class Meta:
        engine = nrm



from coralillo import Model, fields, Engine
from coralillo.auth import PermissionHolder
import unittest

nrm = Engine()


class User(Model, PermissionHolder):
    name = fields.Text()

    class Meta:
        engine = nrm


class PermissionTestCase(unittest.TestCase):

    def setUp(self):
        nrm.lua.drop(args=['*'])
        self.user = User(
            name      = 'juan',
        ).save()
        self.allow_key = self.user.allow_key()

    def test_add_permissions(self):
        self.user.allow('a:b:c')
        self.assertTrue(nrm.redis.sismember(self.allow_key, 'a:b:c'))

    def test_add_ignores_when_has_parent(self):
        self.user.allow('a')
        self.user.allow('a:b')
        self.user.allow('a:b:c')

        self.assertTrue(nrm.redis.sismember(self.allow_key, 'a'))
        self.assertFalse(nrm.redis.sismember(self.allow_key, 'a:b'))
        self.assertFalse(nrm.redis.sismember(self.allow_key, 'a:b:c'))

        self.user.allow('foo:var')
        self.user.allow('foo:var:log')
        self.assertFalse(nrm.redis.sismember(self.allow_key, 'foo'))
        self.assertTrue(nrm.redis.sismember(self.allow_key, 'foo:var'))
        self.assertFalse(nrm.redis.sismember(self.allow_key, 'foo:var:log'))

    def test_revoke_permission(self):
        self.user.allow('a:b')
        self.user.revoke('a:b')

        self.assertFalse(nrm.redis.sismember(self.allow_key, 'a:b'))

    def test_check_permission_inheritance(self):
        self.user.allow('a:b')

        self.assertTrue(self.user.is_allowed('a:b'))
        self.assertTrue(self.user.is_allowed('a:b:c'))

        self.assertFalse(self.user.is_allowed('a'))
        self.assertFalse(self.user.is_allowed('a:d'))

    def test_can_carry_tail(self):
        self.user.allow('org:fleet:view')

        self.assertFalse(self.user.is_allowed('org:fleet:somefleet:view'))
        self.assertTrue(self.user.is_allowed('org:fleet:somefleet', tail='view'))


if __name__ == '__main__':
    unittest.main()



from coralillo import datamodel, Engine, Model, BoundedModel, fields
from coralillo.datamodel import debyte_string
import unittest

nrm = Engine()


class Person(Model):
    name = fields.Text()

    class Meta:
        engine = nrm


class Ship(Model):
    name = fields.Text()
    code = fields.Text(index=True)

    class Meta:
        engine = nrm


class Pet(BoundedModel):
    name = fields.Text()

    @classmethod
    def prefix(cls):
        return 'testing'

    class Meta:
        engine = nrm


class ModelTestCase(unittest.TestCase):

    def setUp(self):
        nrm.lua.drop(args=['*'])

    def test_create_user(self):
        user = Person(
            name      = 'John',
        ).save()

        self.assertEqual(user.name, 'John')

        self.assertTrue(nrm.redis.sismember('person:members', user.id))
        self.assertEqual(nrm.redis.hget('person:{}'.format(user.id), 'name'), b'John')

    def test_retrieve_user_by_id(self):
        carla = Person( name = 'Carla',).save()
        roberta = Person( name = 'Roberta',).save()

        read_user = Person.get(carla.id)

        self.assertEqual(read_user.name, carla.name)

    def test_retrieve_by_index(self):
        titan = Ship(name='te trece', code = 'T13',).save()
        atlan = Ship(name='te catorce', code = 'T14',).save()

        found_ship = Ship.get_by('code', 'T13')

        self.assertIsNotNone(found_ship)
        self.assertTrue(titan == found_ship)

    def test_update_keep_index(self):
        ship = Ship(name='the ship', code='TS').save()

        ship.update(name='updated name')

        self.assertEqual(ship.code, 'TS')
        self.assertEqual(ship.name, 'updated name')

        self.assertEqual(debyte_string(nrm.redis.hget('ship:index_code', 'TS')), ship.id)

    def test_update_changes_index(self):
        ship = Ship(code='THECODE').save()

        ship.update(code='NEWCODE')

        self.assertEqual(ship.code, 'NEWCODE')

        self.assertEqual(debyte_string(nrm.redis.hget('ship:index_code', 'NEWCODE')), ship.id)
        self.assertIsNone(nrm.redis.hget('ship:index_code', 'THECODE'))

    def test_get(self):
        org = Person(name='Juan').save()
        got = Person.get(org.id)

        self.assertTrue(org == got)

    def test_get_all(self):
        p1 = Person(name='Juan').save()
        p2 = Person(name='Pepe').save()

        allitems = Person.get_all()

        allitems.sort(key=lambda x: x.name)

        item1 = allitems[0]
        item2 = allitems[1]

        self.assertEqual(item1, p1)
        self.assertEqual(item2, p2)

    def test_bounded_model(self):
        dev = Pet(
            name = 'foo',
        ).save()

        self.assertFalse(nrm.redis.exists('pet:'+dev.id))
        self.assertTrue(nrm.redis.exists('testing:pet:'+dev.id))
        self.assertEqual(nrm.redis.hget('testing:pet:'+dev.id, 'name'), b'foo')
        self.assertTrue(nrm.redis.sismember('testing:pet:members', dev.id))

    def test_delete(self):
        dev = Pet(
            code = 'foo',
        ).save()

        self.assertIsNotNone(Pet.get(dev.id))

        dev.delete()

        self.assertIsNone(Pet.get(dev.id))
        self.assertFalse(nrm.redis.sismember('testing:pet:members', dev.id))

    def test_delete_index(self):
        ship = Ship(code='A12').save()

        self.assertEqual(debyte_string(nrm.redis.hget('ship:index_code', 'A12')), ship.id)

        ship.delete()

        self.assertFalse(nrm.redis.hexists('ship:index_code', 'A12'))


if __name__ == '__main__':
    unittest.main()
